# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/10 9:38
# @Author   : Fangyang
# @Software : PyCharm


import pandas as pd


class DimensionTable:
    name: str = ""
    db_path: str = ""
    db_name: str = ""
    sort_columns: str = ""
    df: pd.DataFrame = None


if __name__ == '__main__':
    pass
