# !/usr/bin/env python3
# -*- coding:utf-8 -*-
# @Datetime : 2023/10/9 21:21
# @Author   : Fangyang
# @Software : PyCharm

# TODO
# stock 100,5 -> 4328/5529, 240s
# stock 50,5 -> 4328/5529, 619s
AKSHARE_CRAWL_CONCURRENT_LIMIT = 50
AKSHARE_CRAWL_STOP_INTERVEL = 30  # seconds

if __name__ == '__main__':
    pass
